; forth.asm - a TIL implementation of FORTH for Z80
; Copyright (C) 2023  William R. Moore
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.

STACK	EQU	2	; stack pointer
PC	EQU	3	; program counter
CALLR	EQU	4	; call register
RETR	EQU	5	; return register
TXTPTR	EQU	7	; text pointer
INBUF	EQU	8	; Input line buffer pointer register.
; ; 	TIL registers
IP	EQU	10	; Instruction Pointer
WA	EQU	11	; Word Address
TPC	EQU	12	; TIL Program Counter

; Input Restrictions
BUFFER_SIZE EQU 255

CR	EQU 0DH
LF	EQU 0AH
DOLLAR EQU 24H
ESC EQU 27

; Screen print calls
WRITESTR EQU 9H
PRTCHR EQU 02H
BDOS EQU 5H 

  ORG 100h 

MAIN:
  LD DE, GREET
  CALL PUTS

READING:
  CALL INPUT
  JP READING

INPUT:
  LD C,11 ; C_STAT
  LD DE,0
  CALL BDOS
  OR A
  JR Z,INPUT
	
  LD C,6
  LD DE,-1
  CALL BDOS
  OR A
  JP Z,INPUT

  CP 03H
  JP Z, EXIT

  CP 0DH
  JP Z, EXIT

  LD D, A
  LD E, A
  CALL PUTC
  RET

; Print a string
PUTS:
  LD C, WRITESTR
  CALL BDOS
  RET

; Print a character
PUTC:
  LD C, PRTCHR
  CALL BDOS
  RET

CL: DB ESC,"[H",ESC,"[2J$"
BUFFER: DS BUFFER_SIZE
GREET: DB "William Moore's FORTH TIL!", CR, LF, 0H
PROMPT: DB ESC, ESC, "$ ", 0H

EXIT:
  JP 0h

  END